﻿#target bridge
/*-------------------------------------------------------------------------------------------------------------- 
	ADOBE SYSTEMS INCORPORATED
	Copyright 2006 - Adobe Systems Incorporated
	All Rights Reserved
 
	If this file is provided by Adobe in modifiable form (e.g., source code or JavaScript), then you may modify this file.  
	You may use and distribute this file, in modified or unmodified form, provided that you must reproduce the above
	acknowledgement and this notice with all copies of the file.
 
	THIS SOFTWARE IS PROVIDED BY ADOBE SYSTEMS INCORPORATED “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, 
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ADOBE BE LIABLE TO YOU FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
	WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--------------------------------------------------------------------------------------------------------------*/
//----------------------------------------------------------------------- 
// <asm:name>Adobe Library 3</asm:name>
// <asm:description>This file contains dot release patches for AdobeLibraries 1 and 2, as well as a few extra functions.</asm:description>
// <asm:help>None, for developers only</asm:help>
// <asm:author>Bob Stucky</asm:author>
// <asm:company>Adobe Systems, Inc.</asm:company>
// <asm:copyright>(c) Copyright 2004, 2005 Adobe Systems Incorporated, all rights reserved</asm:copyright>
// <asm:version>0.6.11</asm:version>
// <asm:date>10-19-2005</asm:date>
// <asm:website>http://www.adobe.com</asm:website>
//----------------------------------------------------------------------- 
if ( BridgeTalk.appName == "bridge" ) {

/*---------------------------
/*	New Functionality
/*-------------------------*/

getMountedVolumes = function() {
	var v = app.document.thumbnail;
	while ( !getCharsAfter( v.parent, "/" ).startsWith( ":" ) ) {
		v = v.parent;
	}
	return v.children;
}
// functionName - ensureUniqueFolder 
// Description - returns a folder that will point to a UNIQUE name. If the passed folder exists,
// it will append a (n) number to the end of the file (before the extension).
// arguments - folder - any file
// returns - a folder object that is a unique name and will not exist
//
ensureUniqueFolder = function( folder ) {
	var sq = 1;
	var olderFolder = folder;
	while ( folder.exists ) {
		folder = new Folder( olderFolder.path + "/" + olderFolder.name + " (" + sq++ + ")" );
	}
	return folder;
}
// functionName - verifyFolder 
// Description - ensures that a folder passed to it actually exists. If the folder does not exist, it will put up a warning
// that it does not exist and ask if the user wants to create it. If OK clicked, it will create the folder and the entire path
// If the creation of the folder fails, it will return undefined.
// arguments - folder - any file
// returns - a folder object exists, or undefined
//

verifyFolder = function( folder, silent ) {
	if ( !folder.exists ) {
		var str1 = localize( "$$$/WAS/lib3/wordFolder=Folder" ) + " " + decodeURI( folder.fsName );
		var str2 = localize( "$$$/WAS/lib3/confirmFolderCreate=^nDoes not exist, Create?" );
		var makeIt = true;
		if ( !silent ) {
			var makeIt = Window.confirm( str1 + str2 );
		}
		if ( makeIt ) {
			var oFolder = new Folder( folder.absoluteURI );
			var stk = new Stack();
			while ( !oFolder.exists ) {
				stk.push( oFolder );
					oFolder = new Folder( oFolder.path );
			}
			stk2 = new Stack();
			while ( stk.length() > 0 ) {
				oFolder = stk.pop();
				if ( oFolder.create() ) {
					stk2.push( oFolder );
				} else {
					while ( stk2.length() > 0 ) {
						stk2.pop.remove();
					}
					var str3 = localize( "$$$/WAS/lib3/folderNotCreated=^nCould not be created." );
					alert( str1 + str3 );
					return undefined;
				}
			}
			return oFolder;
		} else {
			return undefined;
		}
	} else {
		return folder;
	}
}
isNumber = function( str ) {
	return !isNaN( parseFloat( str ) );
}
isInteger = function( str ) {
	return !isNaN( parseInt( str ) );
}
escapeQuotes = function( st ) {
	st = new String( st );
	st = st.replace( /'/g, "\\'" );
	return st.replace( /"/g, '\\"' );
}
escapeSingleQuotes = function( st ) {
	st = new String( st );
	return st.replace( /'/g, "\\'" );
}
escapeDoubleQuotes = function( st ) {
	st = new String( st );
	return st.replace( /"/g, '\\"' );
}
stripLF = function( s )  {
	if ( isValidReference( s ) )	 {
		var str = new String( s.toString() );
		str = str.replace( /\n/g, "" );
		return str;
	}
	return s;	
}
stripCRLF = function( s ) {
	return stripLF( stripCR( s ) );
}

TYPES.PHOTOSHOP_OPENABLE = "PSD,PDD,JPEG,JPG,JPE,8BPS,GIF,BMP,RLE,DIB,TIF,CRW,NEF,RAF,ORF," +
	"CIN,SDPX,DPX,FIDO,GIF,EPS,PS,EPSF,FLM,PSB,EXR,PCX,PDF,PDP," +
	"PCD,RAW,PICT,PCT,PIC,PXR,PNG,HRR,RGBE,XYZE,TGA,VDA,ICB,VST,TIF,TIFF,WBM,WBMP,DNG,SCT,PBM,CRW,CR2,DC2,DCR,NEF,MRW,X3F,";
/* Renaming support
/*
/* Added ability to use metadata in renaming. */
FileOutputDialog.strings = {};
FileOutputDialog.strings.illegalPathIndex = localize( "$$$/WAS/Lib3/illegalIndex=Illegal Path Index" );
FileOutputDialog.strings.pathIndexOB = localize( "$$$/WAS/Lib3/pathIndexOB=Path index out of bounds" );
FileOutputDialog.strings.supportedVariable = localize( "$$$/WAS/Lib3/unsupported=Unsupported renaming variable" );


FileOutputDialog.parseRenamingTemplate = function( template, file, variableTable, ext, inc )  {
	var incrementSeq = isValidReference( inc ) ? inc : true;
	var s = template;
	var front = getCharsBefore( s, "<var " );
	if ( front == s ) { // means there isn't another "<var..." statement
		return front;
	}
	var junk = getCharsAfter( s, "<var " );
	var key = trim( getCharsBefore( junk, ">" ) );
	var back = getCharsAfter( junk, ">" );
	if ( startsWith( key, "filename" ) ) { // filename is standard in all implementations
		var name = file.getNameWithoutExtension();
		var sub = getCharsAfter( key, "[" );
		var sub = getCharsBefore( sub, "]" );
		if ( isEmpty( sub ) ) {
			sub = "0";
		}
		if ( getCharsBefore( sub, "," ) == sub ) {
			var sub0 = parseInt( sub );
			if ( isNaN( sub0 ) ) {
				throw AdobeLibraryStrings.substringIndexError;
			}
			name = name.substr( sub0 );
			
		} else {
			var sub0 = parseInt( getCharsBefore( sub, "," ) );
			var sub1 = parseInt( getCharsAfter( sub, "," ) );
			if ( isNaN( sub0 ) || isNaN( sub1 ) ) {
				throw AdobeLibraryStrings.substringIndexError;
			}
			name = name.substr( sub0, sub1 );
		}
		s = front + name + back;
		return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );
	} else if ( startsWith( key, "date" ) ) {
		var fmt = getCharsAfter( key, "[" );
		var fmt = getCharsBefore( fmt, "]" );
		s = front + new Date().formatter( fmt ) + back;
		return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );				
	} else if ( startsWith( key, "cdate" ) ) {
		var fmt = getCharsAfter( key, "[" );
		var fmt = getCharsBefore( fmt, "]" );
		s = front + new Date( file.created ).formatter( fmt ) + back;
		return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );		
	} else if ( startsWith( key, "mdate" ) ) {
		var fmt = getCharsAfter( key, "[" );
		var fmt = getCharsBefore( fmt, "]" );
		s = front + new Date( file.modified ).formatter( fmt ) + back;
		return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );
	} else if ( startsWith( key, "metadata" ) ) {
	// format is <metadata[ns:element]>
	// format can now be <metadata[ns:element:date:YYMMDD]>
		var fmt = getCharsAfter( key, "[" );
		var fmt = getCharsBefore( fmt, "]" );
// new code		
		var fmtArray = fmt.split( ":" )
		fmt = fmtArray[ 0 ] + ":" + fmtArray[ 1 ];
// leaves fmt as it was
		var dType = fmtArray[ 2 ];
		var dFmt = fmtArray[ 3 ];
// end of the new code		
		var fmtElement = getCharsAfter( fmt, ":" );
		var tn = new Thumbnail( file );
		var md = tn.synchronousMetadata;
		var mdElement = MdNs.elements.get( fmt );
		var mData =  undefined;
		if ( isValidReference( mdElement ) ) {
			mData =  mdElement.getValue( md );
			if ( !isEmpty( mData ) )  {
// new code, only try this if there's format information
				if ( dType ) {
					switch ( dType ) {
						case "date" : {
							mData = new Date( mData );
							if ( Number( mData.getYear() ) < 2000 ) {
								var year = mData.getYear() + 1000;
								mData.setYear( year );
								mData = mData.formatter( dFmt );
							}
							break;
						}
						default : {
							var x1 = dFmt.split( "," )[0];
							var x2 = dFmt.split( "," )[1];
							if ( isNaN( x1 ) || isNaN( x2 ) ) {
								throw AdobeLibraryStrings.substringIndexError;
							}
							name = name.substr( x1, x2 );
							break;
						}
					}
				}
// end of the new code
// the next replacements eliminate chars that screw up file systems
				mData = replace( new String( mData ), "\/", "-" );
				mData = replace( mData, "\\\\", "-" );	
				mData = replace( mData, ":", "-" );	
			} else {
				mData = localize( "$$$/WAS/noMDAvailable=[null metadata value " ) + fmtElement + "]";
			}
		} else {
			throw localize( "$$$/WAS/lib3/badMDHandle=Invalid Metadata Element" );
		}
		s = front + mData + back;
		return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );
	} else if ( startsWith( key, "sequence" ) ) {
		var fmt = getCharsAfter( key, "[" );
		var fmt = getCharsBefore( fmt, "]" );
		var fmt = parseInt( fmt );
		if ( isNaN( fmt ) || fmt < 1 ) {
			fmt = 3;
		}
		var sqStr = new String( FileOutputDialog.sequenceNumber );
		if ( incrementSeq ) {
			FileOutputDialog.sequenceNumber++;
		}
		var zeros = "0000000000000000000000000000000000000000000000000000000000000000000000000";
		if ( sqStr.length < fmt ) {
			var sq = zeros.substr( 0, ( fmt - sqStr.length ) ) + sqStr;
		} else {
			sq = sqStr;
		}
		s = front + sq + back;
		return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );
	} else if ( startsWith( key, "extension" ) ) {
		if ( !isValidReference( ext ) ) {
			ext = file.getExtension();
			if ( isValidReference( ext ) ) {
				ext = "." + ext;
			} else {
				ext = "";
			}
		} else {
			ext = "." + ext;
		}
		s = front + ext + back;
		return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );
	} else if ( startsWith( key, "path" ) ) { //  so are path elements
		var eText = getCharsAfter( key, "[" );
		var idx = getCharsBefore( eText, "]" );
		var pathArray = new String( File.decode( file.absoluteURI ) ).split( "/" );
		try {
			var e = parseInt( idx );
		} catch ( x ) {
			throw ( FileOutputDialog.strings.illegalPathIndex + ": " + idx );
		}
		var pathElement = null;
		if ( e < 0 )  {
			pathElement = pathArray[ ( pathArray.length + e - 1 ) ];
		} else {
			pathElement = pathArray[ e ];
		}
		if ( !isValidReference( pathElement ) )  {
			throw ( FileOutputDialog.strings.pathIndexOB + ": " + idx );
		}
		s = front + pathElement + back;
		return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );
	} else {
		var repVal = variableTable.get( key );
		if ( isValidReference( repVal ) ) {
			s = front + repVal + back;
			return FileOutputDialog.parseRenamingTemplate( s, file, variableTable, ext, inc );
		} else {
			throw( key + " " + FileOutputDialog.strings.supportedVariable );
		}
	}
}

/*-----------------------------------------------------------------
/*	Metadata Object and Support Stuff
/*----------------------------------------------------------------*/
	KeyedArrayList = function( sorted ) {
		this.sorted = sorted || false;
		this.collection = new Hashtable( true );
	}
	KeyedArrayList.prototype.put = function( key, value ) {
		if ( this.collection.isUniqueKey( key ) ) {
			var ar = new Array();
			ar.push( value );
			this.collection.put( key, ar );
		} else {
			ar = this.collection.get( key );
			for ( i = 0; i < ar.length; i++ ) {
				if ( ar[ i ] == value ) {
					return;
				}
			}
			ar.push( value );
			if ( this.sorted ) {
				ar.sort();
			}
		}
	}
	KeyedArrayList.prototype.get = function( key ) {
		return this.collection.get( key );
	}
	KeyedArrayList.prototype.getKeys = function() {
		return this.collection.keyList;
	}
	
	MdNs = {};

	MdNs.elements = new Hashtable();
	MdNs.namespaces = new Hashtable();
	MdNs.namespacePrefixes = new Hashtable();
	
	MdNs.MdManager = function( thumbnail ) {
		this.thumbnail = thumbnail;
		this.md = thumbnail.metadata;
		this.enumeration = new KeyedArrayList();
		this.noValue = new KeyedArrayList();
		this.notScriptable = new KeyedArrayList();
	}
	MdNs.MdManager.prototype.grab = function() {
		var md = this.thumbnail.metadata;
		var res = new KeyedArrayList();
		var elements = MdNs.elements.keyList;
		for ( var i = 0; i < elements.length; i++ ) {
			var el = MdNs.elements.get( elements[ i ] );
			if ( el.scriptAccessible ) {
				try {
					var val = el.getValue( md );
					if ( isValidReference( val ) && ( val.length > 0 ) && ( trim( val ) != null ) ) {
						var mdValue = new MdNs.MdValue( el.ns, el, val );
						this.enumeration.put( el.ns.common, mdValue );
					} else {
						this.noValue.put( el.ns.common, new MdNs.MdValue( el.ns, el, undefined ) );
					}
				} catch ( e ) {
				}
			} else {
				this.notScriptable.put( el.ns.common, new MdNs.MdValue( el.ns, el, undefined ) );
			}
		}
	}
	MdNs.MdManager.prototype.stuff = function() {
//		var md = this.thumbnail.metadata;
// not implemented
	}
	MdNs.MdManager.prototype.set = function( common, name, value ) {
//		var md = this.thumbnail.metadata;
// not implemented
	}
	MdNs.MdManager.prototype.get = function( common, name ) {
//		var md = this.thumbnail.metadata;
// not implemented
	}
	MdNs.loadMetadata = function() {
		var files = getBridgeThumbnails();
		for ( var i = 0; i < files.length; i++ ) {
			if ( isFile( files[ i ] ) ) {
				var md = new MdNs.MdManager( files[ i ] );
				md.enumeration = md.grab();
				files[ i ].mdManager = md;
			}
		}
	}
	MdNs.MdValue = function( ns, element, val ) {
		this.ns = ns;
		this.element = element;
		this.value = val;
	}
	MdNs.MetadataNamespace = function( common, ns, prefix ) {
		this.common = common;
		this.ns = ns;
		this.handle = prefix;
		MdNs.namespaces.put( common, this );
		MdNs.namespacePrefixes.put( prefix, this );
		this.children = new Array();
	}
	new MdNs.MetadataNamespace( "IPTC Core", "http://iptc.org/std/Iptc4xmpCore/1.0/xmlns/", "Iptc4xmpCore" );
	new MdNs.MetadataNamespace( "EXIF", "http://ns.adobe.com/exif/1.0/", "exif");
	new MdNs.MetadataNamespace( "Dublin Core", "http://purl.org/dc/elements/1.1/", "dc" );
	new MdNs.MetadataNamespace( "TIFF", "http://ns.adobe.com/tiff/1.0/", "tiff" );
	new MdNs.MetadataNamespace( "XMP Basic", "http://ns.adobe.com/xap/1.0/", "xmp"  );
	new MdNs.MetadataNamespace( "XMP Media Management", "http://ns.adobe.com/xap/1.0/mm/", "xmpMM" );
	new MdNs.MetadataNamespace( "XMP Rights Management", "http://ns.adobe.com/xap/1.0/rights/", "xmpRights" );
	new MdNs.MetadataNamespace( "XMP Basic Job Ticket", "http://ns.adobe.com/xap/1.0/bj/", "xmpBJ" );
	new MdNs.MetadataNamespace( "XMP Paged-Text", "http://ns.adobe.com/xap/1.0/t/pg/", "xmpTPg" );
	new MdNs.MetadataNamespace( "Photoshop", "http://ns.adobe.com/photoshop/1.0/", "photoshop" );
//	new MdNs.MetadataNamespace( "Stock Photo", "http://ns.adobe.com/StockPhoto/1.0/", "bmsp" );
	new MdNs.MetadataNamespace( "Adobe PDF", "http://ns.adobe.com/pdf/1.3/", "pdf" );
	new MdNs.MetadataNamespace( "Camera Raw Settings", "http://ns.adobe.com/camera-raw-settings/1.0/", "crs" );
//	new MdNs.MetadataNamespace( "PNG", "http://ns.adobe.com/png/1.0/", "png" );
	
	MdNs.MdElement = function( nsCommon, property, gettable, redirectHandle ) {
		this.nsCommon = nsCommon;
		this.scriptAccessible = gettable;
		this.property = property;
		this.ns = MdNs.namespaces.get( this.nsCommon );
		this.ns.children.push( this );
		this.handle = this.ns.handle + ":" + this.property;
		MdNs.elements.put( this.handle, this );
		this.redirectHandle = redirectHandle;
		this.redirected = false;
		this.redirectElement = undefined;
		if ( isValidReference( this.redirectHandle ) ) {
			this.redirectElement = MdNs.elements.get( this.redirectHandle );
			this.scriptAccessible = this.redirectElement.scriptAccessible;
			this.redirected = true;
		}
	}
	MdNs.MdElement.prototype.getValue = function( md ) {
		if ( !this.scriptAccessible ) {
			throw ( this.nsCommon + ", " + this.property + " " + localize( "$$$/WAS/MD/notscriptable=cannot be accessed via scripting" ) );
		}
		if ( this.redirected ) {
			return this.redirectElement.getValue( md );
		} else {
			md.namespace = this.ns.ns;
			var st = "md." + this.property + ";";
			return eval( st );
		}
	}
	MdNs.MdElement.prototype.toString = function() {
		return "[Object: MdNs.MdElement]";
	}
	
	new MdNs.MdElement( "Dublin Core", "contributor", true ); // false );
	new MdNs.MdElement( "Dublin Core", "coverage", true );
	new MdNs.MdElement( "Dublin Core", "creator", true ); // false );
	new MdNs.MdElement( "Dublin Core", "date", true ); // false );
	new MdNs.MdElement( "Dublin Core", "description", true );
	new MdNs.MdElement( "Dublin Core", "format", true );
	new MdNs.MdElement( "Dublin Core", "identifier", true );
	new MdNs.MdElement( "Dublin Core", "language", true ); // false );
	new MdNs.MdElement( "Dublin Core", "publisher", true ); // false );
	new MdNs.MdElement( "Dublin Core", "relation", true ); // false );
	new MdNs.MdElement( "Dublin Core", "rights", true );
	new MdNs.MdElement( "Dublin Core", "source", true );
	new MdNs.MdElement( "Dublin Core", "subject", true ); // false );
	new MdNs.MdElement( "Dublin Core", "title", true );
	new MdNs.MdElement( "Dublin Core", "type", true ); // false );
	
	new MdNs.MdElement( "XMP Basic", "Advisory", true ); // false );
	new MdNs.MdElement( "XMP Basic", "BaseURL", true );
	new MdNs.MdElement( "XMP Basic", "CreateDate", true );
	new MdNs.MdElement( "XMP Basic", "Identifier", true ); // false );
	new MdNs.MdElement( "XMP Basic", "MetaDataDate", true );
	new MdNs.MdElement( "XMP Basic", "ModifyDate", true );
	new MdNs.MdElement( "XMP Basic", "Nickname", true );
	new MdNs.MdElement( "XMP Basic", "Thumbnail", true ); // false );
	
	new MdNs.MdElement( "XMP Rights Management", "Certificate", true );
	new MdNs.MdElement( "XMP Rights Management", "Marked", true );
	new MdNs.MdElement( "XMP Rights Management", "Owner", true ); // false );
	new MdNs.MdElement( "XMP Rights Management", "UsageTerms", true );
	new MdNs.MdElement( "XMP Rights Management", "WebStatement", true );

	new MdNs.MdElement( "XMP Media Management", "DerivedFrom", true );
	new MdNs.MdElement( "XMP Media Management", "DocumentID", true );
	new MdNs.MdElement( "XMP Media Management", "History", true ); // false );
	new MdNs.MdElement( "XMP Media Management", "ResourceEvent", true ); // false );
	new MdNs.MdElement( "XMP Media Management", "ManagedFrom", true );
	new MdNs.MdElement( "XMP Media Management", "Manager", true );
	new MdNs.MdElement( "XMP Media Management", "ManageTo", true );
	new MdNs.MdElement( "XMP Media Management", "ManageURI", true );
	new MdNs.MdElement( "XMP Media Management", "ManagerVariant", true );
	new MdNs.MdElement( "XMP Media Management", "RenditionClass", true );
	new MdNs.MdElement( "XMP Media Management", "RenditionParams", true );
	new MdNs.MdElement( "XMP Media Management", "VersionID", true );
	new MdNs.MdElement( "XMP Media Management", "Versions", true ); // false );
	new MdNs.MdElement( "XMP Media Management", "LastURL", true );
	
	new MdNs.MdElement( "XMP Basic Job Ticket", "JobRef", true ); // false );
	
	new MdNs.MdElement( "XMP Paged-Text", "MaxPageSize", true );
	new MdNs.MdElement( "XMP Paged-Text", "NPages", true );
	
	new MdNs.MdElement( "Adobe PDF", "Keywords", true );
	new MdNs.MdElement( "Adobe PDF", "Pdfversion", true );
	new MdNs.MdElement( "Adobe PDF", "AgentName", true );
	
	new MdNs.MdElement( "Photoshop", "AuthorsPosition", true );
	new MdNs.MdElement( "Photoshop", "CaptionWriter", true );
	new MdNs.MdElement( "Photoshop", "Category", true );
	new MdNs.MdElement( "Photoshop", "City", true );
	new MdNs.MdElement( "Photoshop", "Country", true );
	new MdNs.MdElement( "Photoshop", "Credit", true );
	new MdNs.MdElement( "Photoshop", "DateCreated", true );
	new MdNs.MdElement( "Photoshop", "Headline", true );
	new MdNs.MdElement( "Photoshop", "Instructions", true );
	new MdNs.MdElement( "Photoshop", "Source", true );
	new MdNs.MdElement( "Photoshop", "State", true );
	new MdNs.MdElement( "Photoshop", "SupplementalCategories", true );
	new MdNs.MdElement( "Photoshop", "TransmissionReference", true );
	new MdNs.MdElement( "Photoshop", "Urgency", true );
	
	new MdNs.MdElement( "TIFF", "ImageWidth", true );
	new MdNs.MdElement( "TIFF", "ImageLength", true );
	new MdNs.MdElement( "TIFF", "BitsPerSample", true ); // false );
	new MdNs.MdElement( "TIFF", "Compression", true ); //1 = uncompressed 6=jpeg
	new MdNs.MdElement( "TIFF", "PhotometricInterpretation", true ); //2 RGB, 6 YCbCr
	new MdNs.MdElement( "TIFF", "Orientation", true );
	new MdNs.MdElement( "TIFF", "SamplesPerPixel", true );
	new MdNs.MdElement( "TIFF", "PlanarConfiguration", true );
	new MdNs.MdElement( "TIFF", "YCbCrSubSampling", true ); // false );
	new MdNs.MdElement( "TIFF", "YCbCrPositioning", true ); //1 centered 2 co-sited
	new MdNs.MdElement( "TIFF", "XResolution", true );
	new MdNs.MdElement( "TIFF", "YResolution", true );
	new MdNs.MdElement( "TIFF", "ResolutionUnit", true ) //2 inches, 3 centimeters
	new MdNs.MdElement( "TIFF", "TransferFunction", true ); // false );
	new MdNs.MdElement( "TIFF", "WhitePoint", true ); // false );
	new MdNs.MdElement( "TIFF", "PrimaryChromaticities", true ); // false );
	new MdNs.MdElement( "TIFF", "YCbCrCoefficients", true ); // false );
	new MdNs.MdElement( "TIFF", "ReferenceBlackWhite", true ); // false );
	new MdNs.MdElement( "TIFF", "DateTime", true );
	new MdNs.MdElement( "TIFF", "ImageDescription", true );
	new MdNs.MdElement( "TIFF", "Make", true );
	new MdNs.MdElement( "TIFF", "Model", true );
	new MdNs.MdElement( "TIFF", "Software", true );
	new MdNs.MdElement( "TIFF", "Artist", true );
	new MdNs.MdElement( "TIFF", "Copyright", true );
	
	new MdNs.MdElement( "EXIF", "ExifVersion", true );
	new MdNs.MdElement( "EXIF", "FlashpixVersion", true );
	new MdNs.MdElement( "EXIF", "ColorSpace", true ); //1 sRGC -32786 uncalibrated
	new MdNs.MdElement( "EXIF", "ComponentsConfiguration", true ); // false );
	new MdNs.MdElement( "EXIF", "CompressedBitsPerPixel", true );
	new MdNs.MdElement( "EXIF", "PixelXDimension", true );
	new MdNs.MdElement( "EXIF", "PixelYDimension", true );
	new MdNs.MdElement( "EXIF", "MakerNote", true );
	new MdNs.MdElement( "EXIF", "UserComment", true );
	new MdNs.MdElement( "EXIF", "RelatedSoundFile", true );
	new MdNs.MdElement( "EXIF", "DateTimeOriginal", true );
	new MdNs.MdElement( "EXIF", "DateTimeDigitized", true );
	new MdNs.MdElement( "EXIF", "ExposureTime", true );
	new MdNs.MdElement( "EXIF", "FNumber", true );
	new MdNs.MdElement( "EXIF", "ExposureProgram", true ); // 0=not defined, 1=manual 2=normal program 3=aperature priority 4=shutter priority 5=creative program 6=action program  7=portrait mode 8= landscape mode
	new MdNs.MdElement( "EXIF", "SpectralSensitivity", true );
	new MdNs.MdElement( "EXIF", "ISOSpeedRatings", true ); // false );
	new MdNs.MdElement( "EXIF", "OECF", true );
	new MdNs.MdElement( "EXIF", "ShutterSpeedValue", true );
	new MdNs.MdElement( "EXIF", "ApertureValue", true );
	new MdNs.MdElement( "EXIF", "BrightnessValue", true );
	new MdNs.MdElement( "EXIF", "ExposureBiasValue", true );
	new MdNs.MdElement( "EXIF", "MaxApertureValue", true );
	new MdNs.MdElement( "EXIF", "SubjectDistance", true );
	new MdNs.MdElement( "EXIF", "MeteringMode", true ); // 0=unk 1=avg 2=center weighted avg 3=spot 4=multi-spot 5=pattern 6=partial 255=other
	new MdNs.MdElement( "EXIF", "LightSource", true ); // 0-unk 1=daylight 2=flourescent 3=tungsten 17=std light A 18=std light B 19=std light C 20=D55 21=D65 22=D75 255=other
	new MdNs.MdElement( "EXIF", "Flash", true );
	new MdNs.MdElement( "EXIF", "FocalLength", true );
	new MdNs.MdElement( "EXIF", "SubjectArea", true ); // false );
	new MdNs.MdElement( "EXIF", "FlashEnergy", true );
	new MdNs.MdElement( "EXIF", "SpatialResponseFrequency", true );
	new MdNs.MdElement( "EXIF", "FocalPlaneXResolution", true );
	new MdNs.MdElement( "EXIF", "FocalPlaneYResolution", true );
	new MdNs.MdElement( "EXIF", "FocalPlaneResolutionUnit", true ); // 2=inches 3=cent
	new MdNs.MdElement( "EXIF", "SubjectLocation", true ); // false );
	new MdNs.MdElement( "EXIF", "ExposureIndex", true );
	new MdNs.MdElement( "EXIF", "SensingMethod", true ); //1=not def 2=one chip color area sensor 3=two chip 4=three chip 5=color sequential area sensor 7=trilinear sensor 8=color sequential linear sensor
	new MdNs.MdElement( "EXIF", "FileSource", true ); // 3(DSC) only choice
	new MdNs.MdElement( "EXIF", "SceneType", true ); // 1(directly photographed image)
	new MdNs.MdElement( "EXIF", "CFAPattern", true );
	new MdNs.MdElement( "EXIF", "CustomRendered", true ); // 0=normal process 1=custom process
	new MdNs.MdElement( "EXIF", "ExposureMode", true ); // 0=auto 1=manual 2=auto bracket
	new MdNs.MdElement( "EXIF", "WhiteBalance", true );// 0=auto 1=manual
	new MdNs.MdElement( "EXIF", "DigitalZoomRatio", true );
	new MdNs.MdElement( "EXIF", "FocalLengthIn35mmFilm", true );
	new MdNs.MdElement( "EXIF", "SceneCaptureType", true ); // 0=std 1=landscape 2=portrait 3=night scene
	new MdNs.MdElement( "EXIF", "GainControl", true ); // 0=none 1=low gain up 2=hight gain up 3=low gain down 4=high gain down
	new MdNs.MdElement( "EXIF", "Contrast", true ); // 0=normal 1=soft 2=hard
	new MdNs.MdElement( "EXIF", "Saturation", true ); // 0=normal 1=low 2=high
	new MdNs.MdElement( "EXIF", "Sharpness", true ); // 0=normal 1=soft 2=hard
	new MdNs.MdElement( "EXIF", "DeviceSettingsDescription", true );
	new MdNs.MdElement( "EXIF", "SubjectDistanceRange", true );// 0=unk 1=macro 2=close 3=distant
	new MdNs.MdElement( "EXIF", "ImageUniqueID", true );
	new MdNs.MdElement( "EXIF", "GPSVersionID", true );
	new MdNs.MdElement( "EXIF", "GPSLatitude", true );
	new MdNs.MdElement( "EXIF", "GPSLongitude", true );
	new MdNs.MdElement( "EXIF", "GPSAltitudeRef", true ); // 0=above SL 2=below SL
	new MdNs.MdElement( "EXIF", "GPSAltitude", true );
	new MdNs.MdElement( "EXIF", "GPSTimeStamp", true );
	new MdNs.MdElement( "EXIF", "GPSSatellites", true );
	new MdNs.MdElement( "EXIF", "GPSStatus", true ); // A=measurement in progress V=measurement s interoperability
	new MdNs.MdElement( "EXIF", "GPSMeasureMode", true ); // 2=2d measurement 3=3d
	new MdNs.MdElement( "EXIF", "GPSDOP", true );
	new MdNs.MdElement( "EXIF", "GPSSpeedRef", true ); // K=kilometers per hour M=miles per hour N=knots
	new MdNs.MdElement( "EXIF", "GPSSpeed", true );
	new MdNs.MdElement( "EXIF", "GPSTrackRef", true ); // T=true M=Magnetic
	new MdNs.MdElement( "EXIF", "GPSTrack", true );
	new MdNs.MdElement( "EXIF", "GPSImgDirectionRef", true ); // T=true M=mag
	new MdNs.MdElement( "EXIF", "GPSImgDirection", true );
	new MdNs.MdElement( "EXIF", "GPSMapDatum", true );
	new MdNs.MdElement( "EXIF", "GPSDestLatitude", true );
	new MdNs.MdElement( "EXIF", "GPSDestLongitude", true );
	new MdNs.MdElement( "EXIF", "GPSDestBearing", true );
	new MdNs.MdElement( "EXIF", "GPSDestDistanceRef", true ); // K=kilometers M=miles N=knots
	new MdNs.MdElement( "EXIF", "GPSDestDistance", true );
	new MdNs.MdElement( "EXIF", "GPSProcessingMethod", true );
	new MdNs.MdElement( "EXIF", "GPSAreaInformation", true );
	new MdNs.MdElement( "EXIF", "GPSDifferential", true ); // 0=without correction 1=correction applied
	
	new MdNs.MdElement( "IPTC Core", "City", undefined, "photoshop:City" );
	new MdNs.MdElement( "IPTC Core", "CopyrightNotice", undefined, "dc:rights" );
	new MdNs.MdElement( "IPTC Core", "Country", undefined, "photoshop:Country" );
	new MdNs.MdElement( "IPTC Core", "CountryCode", true );
	new MdNs.MdElement( "IPTC Core", "Creator", undefined, "dc:creator" );
	new MdNs.MdElement( "IPTC Core", "ContactInfo", true ); // false );
	new MdNs.MdElement( "IPTC Core", "CreatorJobTitle", undefined, "photoshop:AuthorsPosition" );
	new MdNs.MdElement( "IPTC Core", "DateCreated", undefined, "photoshop:DateCreated" );
	new MdNs.MdElement( "IPTC Core", "Description", undefined, "dc:description" );
	new MdNs.MdElement( "IPTC Core", "DescriptionWriter", undefined, "photoshop:CaptionWriter" );
	new MdNs.MdElement( "IPTC Core", "Headline", undefined, "photoshop:headline" );
	new MdNs.MdElement( "IPTC Core", "Instructions", undefined, "photoshop:instructions" );
	new MdNs.MdElement( "IPTC Core", "IntellectualGenre", true );
	new MdNs.MdElement( "IPTC Core", "JobID", undefined, "photoshop:TransmissionReference" );
	new MdNs.MdElement( "IPTC Core", "Keywords", undefined, "dc:subject" );
	new MdNs.MdElement( "IPTC Core", "Location", true );
	new MdNs.MdElement( "IPTC Core", "Provider", undefined, "photoshop:credit" );
	new MdNs.MdElement( "IPTC Core", "Province-State", undefined, "photoshop:state" );
	new MdNs.MdElement( "IPTC Core", "RightsUsageTerms", undefined, "xmpRights:UsageTerms" );
	new MdNs.MdElement( "IPTC Core", "Scene", true ); // false );
	new MdNs.MdElement( "IPTC Core", "Source", undefined, "photoshop:Source" );
	new MdNs.MdElement( "IPTC Core", "SubjectCode", true ); // false );
	new MdNs.MdElement( "IPTC Core", "Title", undefined, "dc:title" );

	new MdNs.MdElement( "Camera Raw Settings", "Version", true );
	new MdNs.MdElement( "Camera Raw Settings", "WhiteBalance", true );
	new MdNs.MdElement( "Camera Raw Settings", "Temperature", true );
	new MdNs.MdElement( "Camera Raw Settings", "Tint", true );
	new MdNs.MdElement( "Camera Raw Settings", "Exposure", true );
	new MdNs.MdElement( "Camera Raw Settings", "AutoShadows", true );
	new MdNs.MdElement( "Camera Raw Settings", "Shadows", true );
	new MdNs.MdElement( "Camera Raw Settings", "Brightness", true );
	new MdNs.MdElement( "Camera Raw Settings", "Contrast", true );
	new MdNs.MdElement( "Camera Raw Settings", "Saturation", true );
	new MdNs.MdElement( "Camera Raw Settings", "Sharpness", true );
	new MdNs.MdElement( "Camera Raw Settings", "LuminanceSmoothing", true );
	new MdNs.MdElement( "Camera Raw Settings", "ColorNoiseReduction", true );
	new MdNs.MdElement( "Camera Raw Settings", "ChromaticAberrationR", true );
	new MdNs.MdElement( "Camera Raw Settings", "ChromaticAberrationB", true );
	new MdNs.MdElement( "Camera Raw Settings", "VignetteAmount", true );
	new MdNs.MdElement( "Camera Raw Settings", "ShadowTint", true );
	new MdNs.MdElement( "Camera Raw Settings", "RedHue", true );
	new MdNs.MdElement( "Camera Raw Settings", "RedSaturation", true );
	new MdNs.MdElement( "Camera Raw Settings", "GreenHue", true );
	new MdNs.MdElement( "Camera Raw Settings", "GreenSaturation", true );
	new MdNs.MdElement( "Camera Raw Settings", "BlueHue", true );
	new MdNs.MdElement( "Camera Raw Settings", "BlueSaturation", true );
	new MdNs.MdElement( "Camera Raw Settings", "ToneCurveName", true );
	new MdNs.MdElement( "Camera Raw Settings", "CameraProfile", true );
	new MdNs.MdElement( "Camera Raw Settings", "HasSettings", true );
	new MdNs.MdElement( "Camera Raw Settings", "HasCrop", true );
	new MdNs.MdElement( "Camera Raw Settings", "RawFileName", true );


/*-----------------------------------------------------------------
/*	Bug Patches for AdobeLibrary1.jsx and AdobeLibrary2.jsx
/*----------------------------------------------------------------*/
// patch library1
ensureUniqueFile = function( file ) {
	var sq = 1;
	oldFile = file;
	file.namingCollision = false;
	while ( file.exists ) {
		file = new File( oldFile.path + "/" + oldFile.getNameWithoutExtension() + "(" + sq++ + ")." + oldFile.getExtension() );
		file.namingCollision = true;
	}
	return file;
}
	unQuote = function( str )  {
		var s = new String( trim( str ) );
		if ( ( s.charAt( 0 ) == '"' ) && ( s.charAt( ( s.length - 1 ) ) == '"' ) )  {
			return s.substring( 1, ( s.length - 1 ) ).toString();			
		}
		return str;		
	}


// patch for ScriptStore, AdobeLibrary2.jsx, evidently putArray and getArray were not tested... grrrr...
	ScriptStore.Store.prototype.putArray = function( type, name, arr ) {
		if ( contains( type, " " ) ) {
			throw localize( "$$$/WAS/Store/err19=Invalid store name. Store Names cannot contain spaces" );
		}
		var typeNode = this.findNode( type );
		if ( !isValidReference( typeNode ) ) {
			typeNode = this.root.addChildNode( type );
		}
		if ( !isValidReference( typeNode ) ) {
			throw "Error saving data";
		}
		var memberNodes = typeNode.findNodes( "member" );
		for ( var i = 0; i < memberNodes.length; i++ ) {
			var nameAttr = memberNodes[ i ].getAttribute( "name" );
			if ( equalsIgnoreCase( nameAttr, name ) ) { // member name already exists, save over it...
				memberNodes[ i ].removeChildren();
				for ( var a = 0; a < arr.length; a++ ) {
					var newNode = memberNodes[ i ].addChildNode( "element", arr[ a ] );
				}
				this.save();
				return;
			}
		} // if this loop terms, no member node with the same name exists
		var memberNode = typeNode.addChildNode( "member" );
		memberNode.addAttribute( "name", name );
		for ( var a = 0; a < arr.length; a++ ) {
			var newNode = memberNode.addChildNode( "element", arr[ a ] );
		}
		this.save();
	}
	ScriptStore.Store.prototype.getArray = function( type, name ) {
		var typeNode = this.findNode( type );
		if ( !isValidReference( typeNode ) ) {
			return undefined;
		}
		var memberNodes = typeNode.findNodes( "member" );
		for ( var i = 0; i < memberNodes.length; i++ ) {
			var nameAttr = memberNodes[ i ].getAttribute( "name" );
			if ( equalsIgnoreCase( nameAttr, name ) ) { 
				var arr = new Array();
				var elementNodes = memberNodes[ i ].findNodes( "element" );
				for ( var a = 0; a < elementNodes.length; a++ ) {
					var eValue = elementNodes[ a ].value;
					arr.push( eValue );
				}
				return arr;
			}
		} // if this loop terms, no member node with the same name exists
		return undefined;
	}
	
	// patch for library1
	Thumbnail.prototype.relativePath = function( base ) {
//	$.level = 1; debugger;
		if ( !isValidReference( base ) ) {
			if ( equalsIgnoreCase( this.location, "versioncue" ) ) {
				if ( contains( this.path, "soap" ) ) {
					return getCharsAfter( this.path, "/soap/" );
				} else {
					return "/soap/";
				}
			} else {
				return this.getFileSpec().absoluteURI;
			}
		} else {
			if ( equalsIgnoreCase( this.location, "versioncue" ) ) {
				return getCharsAfter( decodeURI( this.path ), base );
			} else {
				return this.getFileSpec().getRelativeURI( base );
			}
		}
	}
	isFile = function( thumb ) {
		try	{
			return thumb.resolve().spec instanceof File;
		} catch ( e ) {
			return false;
		}
	}
	isFolder = function( thumb ) {
		try {
			return thumb.resolve().spec instanceof Folder;
		} catch ( e ) {
			return false;
		}
	}
	isHidden = function( thumb ) {
		var f = thumb.resolve();
		try {
			var boo = f.spec.hidden;
		} catch ( e ) {
			return false;
		}
		if ( !boo ) {
			return false;
		} else {
			return startsWith( f.spec.name, "." );
		}
	}
	Thumbnail.prototype.getFileSpec = function() {
		if ( equalsIgnoreCase( this.location, "versioncue" ) ) {
			return this.spec;
		} else if ( equalsIgnoreCase( this.location, "local" ) ) {
			return File( encodeURI( this.path ) );
		} else {
			return undefined;
		}
	}
	thumbnailsToFiles = function( thumbnails ) {
		var files = new Array();
		var paths = new Array();
		for ( var i = 0; i < thumbnails.length; i++ ) {
			paths.push( thumbnails[ i ].path );
			files.push( thumbnails[ i ].getFileSpec() );
			thumbnails[ i ].realFile = files[ i ];
		}
//		if ( vcNodes ) {
			app.preflightFiles( paths );
//		}
		return files;
	}
	getBridgeFiles = function( mask, getFolderChildren, filesOnly, firstOnly ) {
		return thumbnailsToFiles( getBridgeThumbnails( mask, getFolderChildren, filesOnly, firstOnly ) );
	}
	getBridgeThumbnails = function( mask, getFolderChildren, filesOnly, firstOnly )  {
		var store = ScriptStore.open( "libraryGetBridgeFiles" );
		var warnNoSelection = store.get( "alertFlags", "noselection" );
		if ( !isValidReference( warnNoSelection ) ) {
			warnNoSelection = true;
			store.put( "alertFlags", "noselection", "true" );
		} else {
			warnNoSelection = eval( warnNoSelection );
		}
		var warnSomeBadType = store.get( "alertFlags", "badtypes" );
		if ( !isValidReference( warnSomeBadType ) ) {
			warnSomeBadType = true;
			store.put( "alertFlags", "badtypes", "true" );
		} else {
			warnSomeBadType = eval( warnSomeBadType );
		}
		var folderChildren = isValidReference( getFolderChildren ) ? getFolderChildren : true;
		var getFirstFile = isValidReference( firstOnly ) ? firstOnly : false;
		var getFilesOnly = isValidReference( filesOnly ) ? filesOnly : false;
		mask = isValidReference( mask ) ? mask : "";
		var thumbnails = new Array();
		var goodFiles = false;
		var badFiles = false;	
		if ( app.document.selections.length == 0 && warnNoSelection && !firstOnly ) {
			var warning = new DoNoShowAgainWarningAlert( localize( "$$$/WAS/Library/goodFileBadFile=When you do not select any files, ALL files in the active" ), true, localize( "$$$/WAS/Library/goodFileBadFile2=Bridge document are processed. Click Cancel if this is not what you want!" ) );
			var showAgain = warning.show();
			var storeShowAgain = showAgain ? "true" : "false";
			store.put( "alertFlags", "noselection", storeShowAgain );
			if ( warning.actionCancelled ) {
				return thumbnails;
			}
		}
		if ( isValidReference( app.document ) && isValidReference( app.document.thumbnail ) ) {
			var kids = app.document.selections;
			if ( kids.length > 0 ) { // user selected individual thumbnails in the content pane
				for ( var i = 0; i < kids.length ; i++ ) {
					var thumb = kids[ i ].resolve();
					if ( isFile( thumb ) ) {
						if ( thumb.isFileType( mask ) ) {
							goodFiles = true;
							thumbnails.push( thumb );
							if ( getFirstFile ) {
								return thumbnails;
							}
						} else {
							badFiles = true;
						}
					} else if ( isFolder( thumb ) ) {
						if ( folderChildren ) { // if they selected a folder type, go ahead and get the children
							var kids2 = thumb.children;
							for ( var a = 0; a < kids2.length; a++) {
								var thumb2 = kids2[ a ];
								if ( !isHidden( thumb2 ) ) { // again, skip any hidden files
									if ( thumb2.isFileType( mask ) ) {
										goodFiles = true;
										thumbnails.push( thumb2 );
										if ( getFirstFile ) {
											return thumbnails;
										}
									} else {// here we are not looking at downline folders, doing so would start a full hierarchical file scan (a bad thing)
										if ( !isFolder( thumb2 ) ) {
											badFiles = true;
										}
									}
								}
							}
						} else { // unless, of course, we don't want to (getting the children is the default)
							if ( !getFirstFile && !getFilesOnly ) {
								thumbnails.push( thumb );
							}
						}
					} else {
						alert( AdobeLibraryStrings.fsError );
						return new Array();
					}					
				}
			}
			else if ( app.document.thumbnail.children.length > 0 ) { // user did not select any specific thumbnail, look at all thumbnails in the content pane
				var kids = app.document.visibleThumbnails;
				for ( var i = 0; i < kids.length ; i++ ) {
					var thumb = kids[ i ];
					thumb = thumb.resolve();
					if ( !isHidden( thumb ) ) { // because they didn't select anything, it's possible a hidden file will be in there somewhere...
						if ( isFile( thumb ) ) {
							if ( thumb.isFileType( mask ) ) { // only take those that fit the mask
								goodFiles = true;
								thumbnails.push( thumb );
								if ( getFirstFile ) {
									return thumbnails;
								}
							} else {
								badFiles = true;
							}
						} else if ( isFolder( thumb ) ) {
							if ( !getFirstFile && !getFilesOnly ) {
								thumbnails.push( thumb );
							}
						} else {
							throw AdobeLibraryStrings.fsError;
						}
					}
				}
			}
			if ( warnSomeBadType && goodFiles && badFiles ) { // there were both valid and invalid file types selected...
				// do the warning
				var warning = new DoNoShowAgainWarningAlert( localize( "$$$/WAS/Library/goodFileBadFile3=One or more selected files will be skipped because they are not" ), false, localize( "$$$/WAS/Library/goodFileBadFile3a=the right type of file for the requested operation." ) );
				var showAgain = warning.show();
				var storeShowAgain = showAgain ? "true" : "false";
				store.put( "alertFlags", "badtypes", storeShowAgain );
			}
		}
		return thumbnails;
	}
	// get list of files in a folder
	function getFilesinFolderUsingMask( aThumbnail, mask, getFirstFile ) {
		var stopOnFirstFile = isValidReference( getFirstFile ) ? getFirstFile : false;
		var output = [];
		var kids = aThumbnail.children;
		for ( var i=0; i < kids.length; i++) {
			var thumb = kids[ i ];
			if ( !thumb.hidden ) { // again, skip any hidden files
				if ( thumb.isFileType( mask ) ) {
					output.push( thumb );
					if ( stopOnFirstFile ) {
						return output;
					}
				}// here we are not looking at downline folders, doing so would start a full hierarchical file scan (a bad thing)
			}
		}
		return output;
	}
// patch for Liberry 1
	DoNoShowAgainWarningAlert = function( msg, cancel, msg2 ) {
		this.dialog = new Window( "dialog", localize( "$$$/WAS/Library/LibhelpNoteTitle=Help Note" ) );
		this.dialog.orientation = "column";
		
// ITALIAN CHANGE
		if ( isValidReference( msg2 ) ) {
			msg = msg + " " + msg2;
		}
		var txt = this.dialog.add( "statictext", undefined, msg, { multiline: true } );
//	if ( isValidReference( msg2 ) ) {
//		var txt2 = this.dialog.add( "statictext", undefined, msg2, { multiline: true } );
//	}
		this.notice = this.dialog.add( "checkbox", undefined, localize( "$$$/WAS/SM/noShow=Do not show this message again" ) );
		var group = this.dialog.add( "group" );
		this.actionCancelled = false;
		if ( cancel ) {
			this.cancel = group.add( "button", undefined, localize( "$$$/WAS/SM/canx=Cancel" ) );
			this.cancel.dlg = this.dialog;
			this.cancel.wrapper = this;
			this.cancel.alignment = "right";
			this.cancel.onClick = function() {
				this.dlg.close( 2 );
				this.wrapper.actionCancelled = true;
			}
		}
		this.ok = group.add( "button", undefined, localize( "$$$/WAS/SM/okBtn=OK" ) );
		this.ok.alignment = "right";
		this.ok.dlg = this.dialog;
		this.ok.onClick = function() {
		this.dlg.close( 1 );
	}
		this.dialog.center();
	}
	DoNoShowAgainWarningAlert.prototype.show = function() {	
		this.dialog.show();
		return ( !this.notice.value );
	}

	ThumbnailTree.prototype.refresh = function()  {
		this.children = new Array();
		if ( !isValidReference( this.doc ) ) {
			var thumbs = getBridgeThumbnails( this.filter, false ); // get thumbnails selected in bridge, do not head downline
			for ( var i = 0; i < thumbs.length; i++ ) {
				if ( equalsIgnoreCase( thumbs[ i ].location, "versioncue" ) ) {
					this.children.push( new ThumbnailTreeNode( undefined, thumbs[ i ], this.base, this ) );
				} else {
					this.children.push( new FileTreeNode( undefined, thumbs[ i ].spec, this.base, this ) );
				}
			}
		} else {
			if ( this.doc instanceof Array ) {
				for ( var i = 0; i < this.doc.length; i++ ) {
					if  ( ( this.doc[ i ] instanceof Folder ) || ( this.doc instanceof File ) ) {
						this.children.push( new FileTreeNode( undefined, this.doc[ i ], this.base, this ) );
					} else {
						if ( this.doc[ i ] instanceof Thumbnail ) {
							if ( equalsIgnoreCase( this.doc[ i ].location, "versioncue" ) ) {
								this.children.push( new ThumbnailTreeNode( undefined, this.doc[ i ], this.base, this ) );
							} else {
								this.children.push( new FileTreeNode( undefined, this.doc[ i ].spec, this.base, this ) );
							}
						}
					}
				}
			} else if ( ( this.doc instanceof Folder ) || ( this.doc instanceof File ) ) {
				this.children.push( new FileTreeNode( undefined, this.doc, this.base, this ) );
			} else if ( this.doc instanceof Thumbnail ) {
				if ( equalsIgnoreCase( this.doc.location, "versioncue" ) ) {
					this.children.push( new ThumbnailTreeNode( undefined, this.doc, this.base, this ) );
				} else {
					this.children.push( new FileTreeNode( undefined, this.doc.spec, this.base, this ) ) ;
				}
			} else if ( ( this.doc instanceof Document ) ) {
				var thumbs = getBridgeThumbnails( this.filter, false ); // get thumbnails selected in bridge, do not head downline
				for ( var i = 0; i < thumbs.length; i++ ) {
					if ( equalsIgnoreCase( thumbs[ i ].location, "versioncue" ) )  {
						this.children.push( new ThumbnailTreeNode( undefined, thumbs[ i ], this.base, this ) );
					} else {
						this.children.push( new FileTreeNode( undefined, thumbs[ i ].spec, this.base, this ) );
					}
				}
			} else {
				throw "Invalid object";
			}
		}
		if ( this.recursive )  {
			for ( var i = 0; i < this.children.length ; i++ ) { // fill the tree recursively
				this.children[ i ].populate( this.filter );
			} 
		}
		this.addToControl( this.prune ); // fill the list box
	}
	ThumbnailTreeNode.prototype.hasAChildFileSomewhere = function() {
		if ( isFile( this.obj ) ) {
			return true;
		}
		for ( var i = 0; i < this.children.length; i++ ) {
			var result = this.children[ i ].hasAChildFileSomewhere();
			if ( result ) {
				return true;
			}
		}	
		return false;
	}
	ThumbnailTreeNode.prototype.findNode = function( text ) {
		if ( trim( this.text ) == trim( text ) ) {
			return this;
		}
		for ( var i = 0; i < this.children.length; i++ ) {
			var ans = this.children[ i ].findNode( text );
			if ( isValidReference( ans ) ) {
				return ans;
			}
		}
		return undefined;
	}
	ThumbnailTreeNode.prototype.getFileList = function( anArray, paths, includeFolders )  {
		if ( isFile( this.obj ) || includeFolders ) {
			paths.push( this.obj.path );
			if ( equalsIgnoreCase( this.obj.location, "versioncue" ) )  {
				anArray.push( this.obj.spec );
			} else {
				anArray.push( new File( encodeURI( this.obj.path ) ) );	// sending back an array of file or folder objects
			}
		}
		for ( var i = 0; i < this.children.length ; i++ ) { // fill the array by traversing filetreenodes
			anArray = this.children[ i ].getFileList( anArray, paths, includeFolders );
		}
		return anArray;
	}
	ThumbnailTreeNode.prototype.addToControl = function( control, spacer, prune )  {
		if ( isFile( this.obj ) || ( this.hasAChildFileSomewhere() || !prune ) ) {
			control.add( "item", spacer + this.text );
			for ( var i = 0; i < this.children.length ; i++ ) {
				this.children[ i ].addToControl( control, spacer + "  ", prune );
			}
		}
	}
	ThumbnailTreeNode.prototype.populate = function( filter )  {
		for ( var i = 0; i < this.objChildren.length; i ++ ) {
			var thumb =  this.objChildren[ i ];
			if ( !isValidReference( thumb.aliasType ) ) { // skip alias's's's down here in hierarchy - avoid loops...
				if ( thumb.isFileType( filter ) ) {
					var newNode = this.add( thumb );
				} else if ( isFolder( thumb ) ) {
					var newNode = this.add( thumb );
					newNode.populate( filter );	
				}
			}
		}
	}
	DialogUtilities.ExtendedDialog.prototype.show = function() {
		var result = this.control.show();
		app.document.bringToFront();
		switch ( result ) {
			case 1 : return this;
			case 2 : return undefined;
		}
	}
	DialogUtilities.ExtendedControl.prototype.addComplexEdit = function( name, text, preferredSize, creationProps, caption, min, max, capLoc ) {
		if ( !isValidReference( creationProps ) ) {
			creationProps = {};
		}
		creationProps.name = name;
		var uberGruppe = this;
		if ( isValidReference( min ) ) {
			uberGruppe = this.add( "group", ( name + "GroupCol" ) );
			uberGruppe.setOrientation( "column" );
			uberGruppe.setSpacing( 0 );
		}
		var group = uberGruppe.add( "group", ( name + "Group" ) );
	//	group.setAlignment( "fill" );
		if ( capLoc == "top" ) {
			group.setOrientation( "column" );
			group.setAlignChildren( "left" );
		}
		group.setAlignChildren( "fill" );
		var stat = group.add( "statictext", ( name + "Static" ), caption );
		stat.setAlignment( "top" );
		var edit = group.add( "edittext", name, text, preferredSize, creationProps );
		this.captionField = stat;
	//	edit.setAlignment( "fill" );
		this.edit = edit;
		edit.setAlignment( "right" );
		if ( isValidReference( min ) ) {
			var slider = uberGruppe.add( "slider", ( name + "slider" ) );
			this.slider = slider;
			slider.setAlignment( "right" );
			slider.control.minvalue = min;
			slider.control.maxvalue = max;
			slider.setValue( parseInt( text ) );
			edit.setValue( text );
			edit.savedValue = text;
			edit.slider = slider;
			slider.edit = edit;
			slider.control.onChanging = function() {
				this.wrapper.edit.setValue( Math.round( this.value ) );
				this.wrapper.savedValue = Math.round( this.value );
			}
			slider.control.onChange = function() {
				this.wrapper.edit.setValue( Math.round( this.value ) );
				this.wrapper.savedValue = Math.round( this.value );
			}
			edit.control.onChange = function() {
				if ( isNaN( parseInt( this.text ) ) ) {
					this.text = this.wrapper.savedValue;
					this.wrapper.slider.setValue( this.wrapper.savedValue );
				}
	//			debugger;
				if ( parseInt( this.text ) > this.wrapper.slider.control.maxvalue ) {
					this.text = this.wrapper.savedValue;
					this.wrapper.slider.setValue( this.wrapper.savedValue );
				}
				if ( parseInt(this.text) < this.wrapper.slider.control.minvalue ) {
					this.text = this.wrapper.savedValue;
					this.wrapper.slider.setValue( this.wrapper.savedValue );
				}
				this.wrapper.slider.setValue( Math.round( parseInt( this.text ) ) );
				this.wrapper.savedValue = Math.round( parseInt( this.text ) );
			}
			
		}
		return edit;
	}

//QUICK FIX
	File.prototype.getExtension = function( noDecode ) {
		var idx = decodeURIComponent( this.name ).lastIndexOf( "." );
		if ( noDecode ) {
			var idx = this.name.lastIndexOf( "." );
		}
		var ext = undefined;
		if ( idx > -1 ) {
			if ( noDecode ) {
				ext = this.name.substr( ( idx + 1 ) );
			} else {
				ext = decodeURIComponent( this.name ).substr( ( idx + 1 ) );
			}
		}
		return ext;
	}
	File.prototype.getNameWithoutExtension = function( noDecode ) {
		var fName = decodeURIComponent( this.name );
		if ( noDecode ) {
			var fName = this.name;
		}
		var idx = fName.lastIndexOf( "." ); // could be more than 1 . in the file name
		if ( idx > -1 ) {
			fName = fName.substr( 0, idx );
		}
		return fName;
	}	
	File.prototype.getFileWithExtension = function( ext ) {
	var fName = this.getNameWithoutExtension( true );
	var ext1 = this.getExtension();
	if ( isValidReference( ext ) ) {
		ext1 = ext;
	}
	var newPath = this.path + "/" + fName + "." + ext1;
	return new File( newPath );
}

//---------------------------------------------------------
// Class Name - URL - encapsulates a url string with encoded values
// Description 
// 	makes it easy to construct complex urls
// Usage:
//   var u = new URL( "www.myserver.com", "index.html", "http" );
//   u.addProperty( "fred", "flintstone" );
//   var a = u.toString(); <= a is now: http://www.myserver.com./index.html?fred=flinstone
	function URL( svr, obj, prot ) {
		if ( isValidReference( prot ) ) {
			this.protocol = endsWith( prot, "://" ) ? getCharsBefore( prot, "://" ) : prot;
		} else {
			this.protocol = "http";
		}
		this.server = svr;
		this.object = obj;
		this.uri = protocol + "://" + svr + "/" + obj;
		this.props = new Hashtable();
	}			  
// methods
	URL.prototype.toString = function()  {
		var str = this.uri;
		var iter = this.props.keyIterator();
		var first = true;
		while ( iter.hasNext() ) {
			var k = iter.nextItem();
			var v = this.props.get( k );
			if ( first )  {
				str += "?";
				first = false;
			} else  {
				str += "&";
			}
			str += encodeURIComponent( k ) + "=" + encodeURIComponent( v );		
		}
		return str;
		
	}
	URL.prototype.addProperty = function( k, y ) {
		this.props.put( k, v );
	}
	URL.prototype.clearProperties = function()  {
		this.props = new Hashtable();
	}
// end object URL
	Date.prototype._getMonth = function( mo, longName ) {
		var a = [ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" ]
		var aLong = [ "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" ]
		if ( longName ) {
			return aLong[ mo ];
		} else {
			return a[ mo ];
		}		
	}
	Date.prototype._getDay = function( d, longName ) {
		var a = [ "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" ];
		var aLong = [ "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" ];
		if ( longName ) {
			return aLong[ d ];
		} else {
			return a[ d ];
		}		
	}
	Date.prototype.formatterDateString = function( ch, length) {
		var ds = "";
		switch( ch ) {
			case "Y" : {
				ds = new String( this.getFullYear() );
				if ( length > 4 ) {
					ds = "               ".substr( 0, ( length - 4 ) ) + ds;
				} else {
					ds = ds.substr( ( 4 - length ), length );	
				}
				return ds;
			}
			case "M" : {
				if ( length == 3 ) {
					return this._getMonth( this.getMonth(), false );
				} else if ( length > 3 ) {
					return this._getMonth( this.getMonth(), true );
				} else {
					ds = new String( this.getMonth() + 1 );
					if ( ds.length  == 1 ) {
						ds = "0" + ds;
					}
				}
				break;
			}
			case "D" : {
				if ( length == 3 ) {
					return this._getDay( this.getDay(), false );
				} else if ( length > 3 ) {
					return this._getDay( this.getDay(), true );
				} else {
					ds = new String( this.getDate() );
					if ( ds.length  == 1 ) {
						ds = "0" + ds;
					}
				}
				break;
			}
			case "h" : {
				ds =  new String( this.getHours() + 1 );
				if ( ds.length  == 1 ) {
					ds = "0" + ds;
				}
				break;
			}
			case "m" : { 
				 ds =  new String( this.getMinutes() );
				if ( ds.length  == 1 ) {
					ds = "0" + ds;
				}
				break;
			}
			case "s" : { 
				var ds = new String( this.getSeconds() );
				if ( ds.length  == 1 ) {
					ds = "0" + ds;
				}
				break;
			}
			case "S" : { 
				var ds = new String( this.getMilliseconds() );
				if ( ds.length  == 1 ) {
					ds = "0" + ds;
				}
				break;
			}
			case "Z" : {
				return getCharsAfter( this.toString(), "GMT" );
				break;
			}
			case "z" : {
				var s = getCharsAfter( this.toString(), "GMT" );
				var a = s.substr( 1,2 ) + ":" + s.substr( 3,2 );
				return a;
			}
		}
		if ( length > 2 ) {
			ds = "000000000000000".substr( 0, ( length - 2 ) ) + ds;
		} else {
			ds = ds.substr( ( 2 - length ), length );	
		}
		return ds;
	}
	Date.prototype.formatter = function( fmt, token, out ) {
		if ( !isValidReference( out ) ) {
			out = "";
		}
		if ( fmt.length == 0 ) {
			if ( token.length > 0 ) {
				out += this.formatterDateString( token[ 0 ], token.length );
			}
			return out;
		}
		if ( isValidReference( token ) ) { // we're in a second or higher iteration
			var ch = fmt[ 0 ]; // next char in the format
			var fmt = fmt.substr( 1 ); // stuff after char 1 of the format string
			if ( token.length > 0 ) { // there's something in token...
				var tch = token[ 0 ];
				if ( tch == ch ) { // it's another copy of the existing token
					token += ch;
					return this.formatter( fmt, token, out );
				} 
				if ( "ZzYMDhmsS".indexOf( ch ) > -1 ) { // it's a new token character (different from existing token)
					out += this.formatterDateString( tch, token.length );
					token = ch;
					return this.formatter( fmt, token, out );
				} else { // it's not a format token character
					if ( token.length > 0 ) {
						out += this.formatterDateString( tch, token.length );
					}
					out += ch;
					return this.formatter( fmt, "", out );
				}
			} else { // token is empty 
				if ( "ZzYMDhmsS".indexOf( ch ) > -1 ) { //it's a token char
					return this.formatter( fmt, ch, out );
				} else { // not a token char
					out += ch;
					return this.formatter( fmt, token, out );
				}
			}
		} else {
			return this.formatter( fmt, "", out );
		}
		return out;
	}
	function placeExtendedPanel() {
		// actual need is for a recursive scan, so this kicks of the recursive primitive to handle it all
		placeExtendedPanelPrimitive( this, this );
		this.center();
	}
	function getTextHeight( w ) {
		// recursive scan for a static text object to get the string height
		for ( var i = 0; i < w.children.length; i++ ) {
			if ( w.children[ i ].type == "statictext" ) {
				// found one
				return w.children[ i ].size.height;
			} else if ( w.children[ i ].type == "panel" || w.children[ i ].type == "group" ) {
				// re-enter and look again
				return getTextHeight( w.children[ i ] );
			}
		}
		// return a default value cuz there are no static text elements. 13 is typical value on Windows
		// to do: check platform and return proper default
		return 13;
	}
	function placeExtendedPanelPrimitive( w, c, tHeight ) {
		var crunchToHere = 0;
		var textHeight = 0;
		var textHeight = isValidReference( tHeight ) ? tHeight : getTextHeight( w );

		for ( var i = 0; i < c.children.length; i++ ) {
			if ( c.children[ i ].type == "panel" ) {
				placeExtendedPanelPrimitive( w, c.children[ i ], textHeight );
				if ( isValidReference( c.children[ i ].extendedPanelControl ) ) {
					c.children[ i ].extendedPanelControl.location.x = c.children[ i ].location.x + 17;
					var crunchSpace = c.spacing + c.children[ i ].extendedPanelControl.size.height;
					crunchSpace -= ( ( c.children[ i ].extendedPanelControl.size.height ) / 2  );
					crunchToHere += crunchSpace;					
					c.children[ i ].location.y -= crunchToHere;
					c.children[ i ].parent.size.height -= crunchSpace;
					w.size.height -= crunchSpace;
				} else {
					c.children[ i ].location.y -= crunchToHere;
				}
			} else if ( c.children[ i ].type == "group" ) {
				placeExtendedPanelPrimitive( w, c.children[ i ], textHeight );
			} else {
				c.children[ i ].location.y -= crunchToHere;
			}
//			w.size.height -= crunchSpace;
		}
	}

getCheckboxPanel = function( window, container, txt, enabled ) {
	var box = container.add( "checkbox", undefined, txt );
	var panel = container.add( "panel" );
	panel.value = box.value;
	panel.type = "checkboxpanel";
	window.onShow = placeExtendedPanel;
	panel.extendedPanelControl = box;
	panel.box = box;
	box.panel = panel;
	box.onClick = function( args ) {
		this.panel.enabled = this.value;
		box.panel.value = this.value;
		if ( isValidReference( this.onClickEvent ) ) {
			this.onClickEvent( args );
		}
	}
	box.value = enabled;
	panel.notify = function() {
		this.box.notify();
	}
	panel.synchValue = function( vbl ,ov, nv ) {
		this.panel.value = nv;
	}
	box.watch( "value", panel.synchValue );
	return panel;
}


getDropdownPanel = function( window, container, enabled ) {
	var dropdown = container.add( "dropdownlist" );
	var panel = container.add( "panel" );
	panel.type = "dropdownpanel";
	window.onShow = placeExtendedPanel;
	panel.extendedPanelControl = dropdown;
	panel.dropdown = dropdown;
	dropdown.panel = panel;
//	dropdown.onChange = function() {
//	 	this.panel.selection = this.selection;
//	}
	panel.synchValue = function( vbl ,ov, nv ) {
		this.panel.selection = nv;
	}
	dropdown.watch( "selection", panel.synchValue );
	return panel;
}

	sendBridgeTalkMessage = function( target, script, onResultHandler ) {
		try {
			var bt = new BridgeTalk();
			bt.target = target;
			bt.body = script;
			if (!BridgeTalk.isRunning( target ) )  {
				if ( !startTargetApplication( target ) ) {
					return false;
				}
			}
		} catch (a ) {
			alert( a );
		}
		// bt message interaction handlers

		bt.onResult = onResultHandler;

		bt.onError = function( r )  {
			alert( TranslateErrorCodes.getMessage( r ) );
		}
		return bt.send();
	}

Log = function( logFilePath, includeDate, fmt ) {
	this.includeDate = includeDate || false;
	this.dateFormat = fmt || "MM/DD/YY hh:mm:ss";
	this.file = new File( logFilePath );
	this.file = ensureUniqueFile( this.file );
	if ( this.file.exists ) {
		this.file.open( "e" );
		while ( !this.file.eof ) {
			this.logText += this.file.read( 1 );
		}
	} else {
		this.file.open( "w" );
	}
	this.file.writeln( new Date().formatter( this.dateFormat ) + ": Log Session Opened");
}
Log.prototype.write = function( txt ) {
	txt = txt == undefined ? "" : txt;
	this.file.write( txt );
}
Log.prototype.writeln = function( txt ) {
	txt = txt == undefined ? "" : txt;
	if ( this.includeDate ) {
		if ( txt == "" ) {
			this.file.writeln( txt );
		} else {
			this.file.writeln( new Date().formatter( this.dateFormat ) + ": " + txt );
		}
	} else {
		this.file.writeln( txt );
	}
}
Log.prototype.writeFormatted = function( array ) {
	if ( this.includeDate ) {
		this.file.write( new Date().formatter( this.dateFormat ) + ": " );
	} 
	for ( var i = 0; i < array.length; i++ ) {
		this.file.write( array[ i ] + "\t" );
	}
	this.file.writeln();
}
Log.prototype.close = function() {
	this.file.writeln( new Date().formatter( this.dateFormat ) +  ": Log Session Closed" );
	this.file.writeln();
	this.file.close();
}
FIFOBuffer = function() {
	this.buffer = new Array();
	this.length = 0;
}
FIFOBuffer.prototype.push = function( obj ) {
	this.buffer.push( obj );
	this.length = this.buffer.length;
}
FIFOBuffer.prototype.pop = function() {
	if ( this.buffer.length == 0 ) {
		return undefined;
	}
	var rval = this.buffer[ 0 ];
	this.buffer = arrayRemove( this.buffer, 0 );
	this.length = this.buffer.length;
	return rval;
}
FIFOBuffer.prototype.peek = function() {
	return this.buffer[ 0 ];
}
FIFOBuffer.prototype.toString = function() {
	return "[Object FIFOBuffer]";
}

// new class def for Stack
	function Stack(  ) {
		this.stk = new Array( 0 );
	}
	// pushes an object onto the stack
	Stack.prototype.push = function( st ) {
		this.stk.push( st );
	}
	// pops one off
	Stack.prototype.pop = function() {
	// create a new array 
		var a = new Array( 0 );
		for (var i = 0; i < ( this.stk.length - 1 ) ;i++ ) {
			a[i] = this.stk[i];
		}
		var rVal = this.stk.pop();
		this.stk = a;
		return rVal;
	}
	// returns the index of a value in the stack, -1 if not there
	Stack.prototype.search = function( x )  {
		return getArrayIndex( this.stk, x );
	}
	Stack.prototype.toString = function() {
		return "[object Stack]";
	}
	Stack.prototype.length = function( )  {
		return this.stk.length;
	}
	Stack.prototype.peek = function() {
		return this.stk[ ( this.stk.length - 1 ) ];
	}

ThumbnailTree = function( control, doc, recursive, prune, filter ) {
	this.control = control || undefined;
	this.doc = doc || undefined;
	this.recursive = recursive || false;
	this.prune = prune || false;
	this.filter = filter || undefined;
	if ( isValidReference( app.document ) ) {
		this.base = File.decode( app.document.thumbnail.resolve().relativePath() );
	}
	this.children = undefined;
}

ThumbnailTree.prototype.refresh = function()  {
//$.level = 1;
//debugger;
	this.children = new Array();
	if ( !isValidReference( this.doc ) ) {
		var thumbs = getBridgeThumbnails( this.filter, false ); // get thumbnails selected in bridge, do not head downline
		for ( var i = 0; i < thumbs.length; i++ ) {
			if ( equalsIgnoreCase( thumbs[ i ].location, "versioncue" ) ) {
				this.children.push( new ThumbnailTreeNode( undefined, thumbs[ i ], this.base, this ) );
			} else {
				this.children.push( new FileTreeNode( undefined, thumbs[ i ].spec, this.base, this ) );
			}
		}
	} else {
		if ( this.doc instanceof Array ) {
			for ( var i = 0; i < this.doc.length; i++ ) {
				if  ( ( this.doc[ i ] instanceof Folder ) || ( this.doc instanceof File ) ) {
					this.children.push( new FileTreeNode( undefined, this.doc[ i ], this.base, this ) );
				} else {
					if ( this.doc[ i ] instanceof Thumbnail ) {
						if ( equalsIgnoreCase( this.doc[ i ].location, "versioncue" ) ) {
							this.children.push( new ThumbnailTreeNode( undefined, this.doc[ i ], this.base, this ) );
						} else {
							this.children.push( new FileTreeNode( undefined, this.doc[ i ].spec, this.base, this ) );
						}
					}
				}
			}
		} else if ( ( this.doc instanceof Folder ) || ( this.doc instanceof File ) ) {
			this.children.push( new FileTreeNode( undefined, this.doc, this.base, this ) );
		} else if ( this.doc instanceof Thumbnail ) {
			if ( equalsIgnoreCase( this.doc.location, "versioncue" ) ) {
				this.children.push( new ThumbnailTreeNode( undefined, this.doc, this.base, this ) );
			} else {
				this.children.push( new FileTreeNode( undefined, this.doc.spec, this.base, this ) ) ;
			}
		} else if ( ( this.doc instanceof Document ) ) {
			var thumbs = getBridgeThumbnails( this.filter, false ); // get thumbnails selected in bridge, do not head downline
			for ( var i = 0; i < thumbs.length; i++ ) {
				if ( equalsIgnoreCase( thumbs[ i ].location, "versioncue" ) )  {
					this.children.push( new ThumbnailTreeNode( undefined, thumbs[ i ], this.base, this ) );
				} else {
					this.children.push( new FileTreeNode( undefined, thumbs[ i ].spec, this.base, this ) );
				}
			}
		} else {
			throw "Invalid object";
		}
	}
//	if ( this.recursive )  {
		for ( var i = 0; i < this.children.length ; i++ ) { // fill the tree recursively
			this.children[ i ].populate( this.filter, this.recursive );
		} 
//	}
	this.addToControl( this.prune ); // fill the list box
}
ThumbnailTree.prototype.addToControl = function( prune ) {
	if ( isValidReference( this.control ) ) {
		this.control.selection = null;
		this.control.removeAll();
		for ( var i = 0; i < this.children.length ; i++ ) { // fill the listbox
			this.children[ i ].addToControl( this.control, "", prune );
		}
	}
}
ThumbnailTree.prototype.removeSelection = function( selection ) {
	var sel = selection;
	if ( ( sel instanceof Array ) ) {
		sel = [ sel ];
	}
	for ( var i = 0; i < sel.length; i++ ) {
		this.remove( trim( sel[ i ] ) );
	}
	this.addToControl( this.prune );
}
ThumbnailTree.prototype.remove = function( text )  { //selection is an array of list items
	for ( var i = 0; i < this.children.length ; i++ ) { // against each child
		if ( trim( this.children[ i ].text ) == text )  {
			this.children = arrayRemove( this.children, i ); // remove the bad child
			break;  // since these should be unique, return
		} else {
			if ( this.children[ i ].remove( text ) )  {
				break;	
			} // if not this child, check progeny
		}
	}
}
ThumbnailTree.prototype.removeSingleton = function( selection )  {
	this.remove( trim( selection.text ) );
	this.addToControl( this.prune );
}
ThumbnailTree.prototype.move = function( selection, up ) {
// note this assumes there is only one selected item
	if ( selection.length > 1 ) {
		throw AdobeLibraryStrings.MoveOneError;
	}
	for ( var i = 0; i < this.children.length ; i++ ) { // against each child
	// these children are top level hierarchy, can't move up,
		if ( this.children[ i ].text == selection.text )  {
			if ( up ) {
				if ( i > 0 ) {
					this.children = arrayMove( this.children, i, ( i - 1 ) ); // move up
				}
			} else {
//			debugger;
				if ( i < ( this.children.length - 1 ) ) {
					this.children = arrayMove( this.children, i, ( i + 1 ) ); // move down
				}
			}
			break;  // since these should be unique, return
		} else {
			if ( this.children[ i ].move( selection.text, up ) )  {
				break;	
			} // if not this child, check progeny
		}
	}
	this.addToControl();
}
ThumbnailTree.prototype.getFileList = function( includeFolders )  {
	var anArray = new Array();
	var pathList = new Array();
	for ( var i = 0; i < this.children.length ; i++ ) { // fill the listbox
		anArray = this.children[ i ].getFileList( anArray, pathList, includeFolders );
	}
	app.preflightFiles( pathList );
	return anArray;
}
ThumbnailTree.prototype.findSelectedNode = function( selection ) {
	return this.findNode( trim( selection.text ) );
}
ThumbnailTree.prototype.findNode = function( text ) {
	for ( var i = 0; i < this.children.length; i++ ) {
		if ( this.children[ i ].text == text ) {
			return this.children[ i ];
		} else {
			var ans = this.children[ i ].findNode( text );
			if ( isValidReference( ans ) ) {
				return ans;
			}
		}
	}
	return undefined;
}

ThumbnailTreeNode = function( parent, obj, baseDir, fileTree ) {
//$.level =1;
//debugger;
	this.parent = parent || undefined;
	this.obj = obj || undefined;
	this.base = baseDir || undefined;
	this.tree = fileTree || undefined;
//	this.suffix = this.thumbnail.isFileNode() ? "  (file)" : "  (folder)";
	if ( isValidReference( this.obj ) ) {
		this.text = File.decode( this.relativePath( this.base ) );
	}
	this.children = new Array();
//	this.cacheChildren();
}
ThumbnailTreeNode.prototype.relativePath = function( base )  {
	return this.obj.relativePath( base );
}
ThumbnailTreeNode.prototype.cacheChildren = function()  {
	if ( isValidReference( this.obj ) ) {
		if ( this.obj instanceof Thumbnail )  {
			this.objChildren = this.obj.children;
		} else {
			if ( this.obj instanceof Folder )  {
				this.objChildren = this.obj.getFiles();
			}
		}
	}
}
ThumbnailTreeNode.prototype.hasAChildFileSomewhere = function() {
	if ( AdobeLibrary1.isFile( this.obj ) ) {
		return true;
	}
	for ( var i = 0; i < this.children.length; i++ ) {
		var result = this.children[ i ].hasAChildFileSomewhere();
		if ( result ) {
			return true;
		}
	}	
	return false;
}
ThumbnailTreeNode.prototype.findNode = function( text ) {
	if ( trim( this.text ) == trim( text ) ) {
		return this;
	}
	for ( var i = 0; i < this.children.length; i++ ) {
		var ans = this.children[ i ].findNode( text );
		if ( isValidReference( ans ) ) {
			return ans;
		}
	}
	return undefined;
}
ThumbnailTreeNode.prototype.getFileList = function( anArray, paths, includeFolders )  {
	if ( AdobeLibrary1.isFile( this.obj ) || includeFolders ) {
		paths.push( this.obj.path );
		anArray.push( new File( encodeURI( this.obj.spec ) ) );	// sending back an array of file or folder objects
	}
	for ( var i = 0; i < this.children.length ; i++ ) { // fill the array by traversing filetreenodes
		anArray = this.children[ i ].getFileList( anArray, paths, includeFolders );
	}
	return anArray;
}
ThumbnailTreeNode.prototype.addToControl = function( control, spacer, prune )  {
	if ( AdobeLibrary1.isFile( this.obj ) || ( this.hasAChildFileSomewhere() || !prune ) ) {
		control.add( "item", spacer + this.text );
		for ( var i = 0; i < this.children.length ; i++ ) {
			this.children[ i ].addToControl( control, spacer + "  ", prune );
		}
	}
}
ThumbnailTreeNode.prototype.populate = function( filter, recursive )  {
	if ( isValidReference( this.obj ) ) {
		if ( this.obj instanceof Thumbnail )  {
			this.objChildren = this.obj.children;
		} else {
			if ( this.obj instanceof Folder )  {
				this.objChildren = this.obj.getFiles();
			}
		}
	}
	for ( var i = 0; i < this.objChildren.length; i ++ ) {
		var thumb =  this.objChildren[ i ];
		if ( !isValidReference( thumb.aliasType ) ) { // skip alias's's's down here in hierarchy - avoid loops...
			if ( thumb.isFileType( filter ) ) {
				var newNode = this.add( thumb );
			} else if ( AdobeLibrary1.isFolder( thumb ) ) {
				var newNode = this.add( thumb );
				if ( recursive ) {
					newNode.populate( filter );	
				}
			}
		}
	}
}
ThumbnailTreeNode.prototype.add = function( thumbnail )  {
	var newNode = new ThumbnailTreeNode( this, thumbnail, this.base, this.tree );
	this.children.push( newNode );
	return newNode;
}
ThumbnailTreeNode.prototype.remove = function( text )  { // returning true stops the loop the remove in FileNode.remove()
	for ( var i = 0; i < this.children.length ; i++ ) {
		if ( trim( this.children[ i ].text ) == trim( text ) )  {
			this.children = arrayRemove( this.children, i );
			return true;
		} else {
			if ( this.children[ i ].remove( text ) )  {
				return true;
			}
		}
	}
	return false;
}
ThumbnailTreeNode.prototype.move = function( text, up )  { // returning true stops the loop the remove in FileNode.remove()
	for ( var i = 0; i < this.children.length ; i++ ) {
		if ( trim( this.children[ i ].text ) == trim( text ) )  {
			if ( up ) {
				if ( i > 0 ) {
					this.children = arrayMove( this.children, i, ( i - 1 ) );
				}
			} else {
				if ( i < ( this.children.length - 1 ) ) {
					this.children = arrayMove( this.children, i, ( i + 1 ) );
				}
			}
			return true;
		} else {
			if ( this.children[ i ].move( text, up ) )  {
				return true;
			}
		}
	}
	return false;
}


FileTree = function( control, doc, recursive, prune, filter )  {
	this.ancestor = ThumbnailTree;
	this.ancestor( control, doc, recursive, prune, filter );
}
FileTree.prototype = new ThumbnailTree;
// end class FileTree

//---------------------------------------------------------
// Class Name - FileTreeNode - Helps FileTree do it's thing
//              
// Description 
//   you should never have to use this class by itself
//

FileTreeNode = function( parent, obj, baseDir, fileTree )  {
	this.ancestor = ThumbnailTreeNode;
	this.ancestor( parent, obj, baseDir, fileTree );
}
FileTreeNode.prototype = new ThumbnailTreeNode;

FileTreeNode.prototype.relativePath = function( base )  {
	var sufx = ( this.obj instanceof Folder ) ? "/" : "";
	return File.decode( this.obj.getRelativeURI( this.base ) + sufx );
}

FileTreeNode.prototype.hasAChildFileSomewhere = function() {
	if ( this.obj instanceof File ) {
		return true;
	}
	for ( var i = 0; i < this.children.length; i++ ) {
		var result = this.children[ i ].hasAChildFileSomewhere();
		if ( result ) {
			return true;
		}
	}	
	return false;
}
FileTreeNode.prototype.getFileList = function( anArray, paths, includeFolders )  {
	if ( ( this.obj instanceof File ) || includeFolders ) {
//		paths.push( this.obj.absoluteURI );
		anArray.push( this.obj );	// sending back an array of file or folder objects
	}
	for ( var i = 0; i < this.children.length ; i++ ) { // fill the array by traversing filetreenodes
		anArray = this.children[ i ].getFileList( anArray, paths, includeFolders );
	}
	return anArray;
}
FileTreeNode.prototype.populate = function( filter, recursive )  {
	if ( this.obj instanceof Folder )  {  // files have no children...
		var files = this.obj.getFiles(); // get the files it contains
	  	for ( var i = 0; i < files.length; i++ ) { 
	  		if ( files[ i ] instanceof File ) {
				if ( files[ i ].isFileType( filter ) && !files[ i ].alias )  {
					var tn = this.add( files[ i ] );
				}
	 		} else { // itsa folder
	 			var tn = this.add( files[ i ] ); // add a child node to this node
	 			if ( recursive ) {
					tn.populate( filter ); // populate the new node - note if it's a file it's a quick return!
				}
	  		}
	  	}
	}
}
FileTreeNode.prototype.add = function( obj )  {
	var tn = new FileTreeNode( this, obj, this.base, this.tree );
	this.children.push( tn );
	return tn;
}
FileTreeNode.prototype.addToControl = function( control, spacer, prune )  {
	if ( ( this.obj instanceof File ) || ( this.hasAChildFileSomewhere() || !prune ) ) {
		control.add( "item", spacer + this.text );
		for ( var i = 0; i < this.children.length ; i++ ) {
			this.children[ i ].addToControl( control, spacer + "  ", prune );
		}
	}
}

		File.prototype.getExtension = function() {
			return ( (RegExp.__tmp = this.fsName.match(/\.([^\.\/\\]+$)/)) ? RegExp.__tmp[1] : undefined )
		}
		File.prototype.getNameWithoutExtension = function( noDecode ) {
			var fName = decodeURIComponent( this.name );
			if ( noDecode ) {
				var fName = this.name;
			}
			var ext = this.getExtension();
			if ( !ext ) { // there is no extension
				return fName;
			}
			var idx = fName.lastIndexOf( ext ); // could be more than 1 . in the file name
			if ( idx > -1 ) {
				fName = fName.substr( 0, ( idx - 1 ));
			}
			return fName;
		}	
		File.prototype.getFileWithExtension = function( ext ) {
			var fName = this.getNameWithoutExtension( true );
			var newPath = this.path + "/" + fName + "." + ext;
			if ( !ext ) {
				newPath = this.path + "/" + fName;
			}
			return new File( newPath );
		}

AdobeLibraryStrings.fsError = localize( "$$$/WAS/Library/local=This script can not operate from this location" );





}